@@ -1,7 +1,7 @@
 /*
  * Miscellaneous routines for Mathomatic.
  *
- * Copyright (C) 1987-2006 George Gesslein II.
+ * Copyright (C) 1987-2007 George Gesslein II.
  */
 
 #include "includes.h"
@@ -169,7 +169,7 @@ long	*vp;
 }
 
 /*
- * Allocate or reuse an equation space.
+ * Allocate or reuse an empty equation space.
  *
  * Returns new equation space number or -1 on error.
  */
@@ -223,6 +223,18 @@ next_espace()
 	return i;
 }
 
+/*
+ * Copy equation number "src" to equation number "dest".
+ */
+copy_espace(src, dest)
+int	src, dest;
+{
+	blt(lhs[dest], lhs[src], n_lhs[src] * sizeof(token_type));
+	n_lhs[dest] = n_lhs[src];
+	blt(rhs[dest], rhs[src], n_rhs[src] * sizeof(token_type));
+	n_rhs[dest] = n_rhs[src];
+}
+
 /*
  * Return true if equation number "i" is solved for a normal variable.
  */
@@ -331,7 +343,7 @@ int		n;		/* expression length */
 }
 
 /*
- * This is called when the max expression size has been exceeded.
+ * This is called when the maximum expression size has been exceeded.
  *
  * There is no return.
  */
@@ -356,7 +368,7 @@ char	*cp;
 		i = cur_equation;
 	} else {
 		i = decstrtol(cp, &cp) - 1;
-		if (extra_garbage(cp))
+		if (extra_characters(cp))
 			return -1;
 	}
 	if (not_defined(i)) {
@@ -407,6 +419,8 @@ char		*cp;		/* string to parse */
 			blt(equation, lhs[i], n_lhs[i] * sizeof(token_type));
 			*np = n_lhs[i];
 		}
+		list_proc(equation, *np, false);
+		fprintf(gfp, "\n");
 		return cp2;
 	}
 	return parse_section(equation, np, cp);
@@ -434,7 +448,7 @@ int		*np;		/* pointer to the returned parsed expression length */
 			str_tolower(cp);
 		}
 		cp = parse_expr(equation, np, cp);
-		if (cp && !extra_garbage(cp)) {
+		if (cp && !extra_characters(cp)) {
 			break;
 		}
 	}
@@ -462,7 +476,7 @@ long	*vp;	/* pointer to the returned variable */
 			return false;
 		}
 		cp = parse_var2(vp, cp);
-		if (cp == NULL || extra_garbage(cp)) {
+		if (cp == NULL || extra_characters(cp)) {
 			continue;
 		}
 		return true;
@@ -698,11 +712,11 @@ int	*ip, *jp;
  * on a command line.
  *
  * Returns false if OK.
- * Returns true if non-space characters were encountered
+ * Returns true if any non-space characters were encountered
  * and an error message was printed.
  */
 int
-extra_garbage(cp)
+extra_characters(cp)
 char	*cp;	/* command line */
 {
 	if (cp) {
@@ -726,7 +740,7 @@ int	*ip, *jp;
 	if (!get_range(cpp, ip, jp)) {
 		return false;
 	}
-	if (extra_garbage(*cpp)) {
+	if (extra_characters(*cpp)) {
 		return false;
 	}
 	return true;
