@@ -1,7 +1,7 @@
 /*
  * Main include file for Mathomatic.
  *
- * Copyright (C) 1987-2006 George Gesslein II.
+ * Copyright (C) 1987-2007 George Gesslein II.
  */
 
 #define	true	1
@@ -61,6 +61,8 @@
 
 /*
  * The following defines the default maximum mathematical expression size.
+ * Expression arrays are allocated with this size by default.
+ * It is linearly related to the actual memory usage of Mathomatic.
  * This should be made much smaller for Palmtops or embedded systems.
  * Do not set to less than 100.
  */
@@ -74,7 +76,7 @@
 #define	MAX_VARS	500			/* maximum number of unique variables handled in each equation */
 
 #define	VAR_OFFSET	'A'			/* makes space for predefined variables */
-#define	VAR_MASK	0x3fffL			/* mask for bits containing the variable name */
+#define	VAR_MASK	0x3fffL			/* mask for bits containing a reference to the variable name */
 #define	VAR_SHIFT	14			/* number of bits set in VAR_MASK */
 #define	SUBSCRIPT_MASK	63			/* mask for variable subscript after shifting VAR_SHIFT */
 #define	MAX_SUBSCRIPT	(SUBSCRIPT_MASK - 1)	/* maximum variable subscript */
@@ -90,12 +92,12 @@ enum kind_list {		/* kinds of tokens specified in the union below */
 typedef union {
 	double	constant;	/* internal storage for mathematical constants */
 	long	variable;	/* internal storage for mathematical variables */
-/* predefined variables follow (order is important) */
+/* predefined special variables follow (order is important) */
 #define	V_NULL		0L	/* null variable (display nothing) */
-#define	V_E		1L	/* the constant "e" */
+#define	V_E		1L	/* the constant "e" or "e#" */
 #define	V_PI		2L	/* the constant "pi" */
-#define	IMAGINARY	3L	/* the imaginary constant "i#" */
-#define	SIGN		4L	/* the "sign" variable */
+#define	IMAGINARY	3L	/* the imaginary constant "i" or "i#" */
+#define	SIGN		4L	/* for "sign" variables */
 #define	MATCH_ANY	5L	/* match any variable (wild-card variable) */
 	int	operatr;	/* internal storage for mathematical operators */
 /* valid operators follow (order doesn't matter) */
