@@ -1,7 +1,7 @@
 /*
  * Mathomatic commands that don't belong anywhere else.
  *
- * Copyright (C) 1987-2006 George Gesslein II.
+ * Copyright (C) 1987-2007 George Gesslein II.
  */
 
 #include "includes.h"
@@ -28,10 +28,6 @@ int
 version_cmd(cp)
 char	*cp;	/* the command line argument */
 {
-	if (extra_garbage(cp)) {
-		/* if anything on the command line, fail */
-		return false;
-	}
 #if	LIBRARY
 	result_str = strdup(VERSION);
 #endif
@@ -104,7 +100,7 @@ char	*cp;
 	if ((cp = parse_equation(i, cp)) != NULL) {
 		if (n_lhs[i] && n_rhs[i]) {
 			error(_("Invalid argument."));
-		} else if (!extra_garbage(cp) && solve(i, cur_equation)) {
+		} else if (!extra_characters(cp) && solve(i, cur_equation)) {
 			return return_result(cur_equation);
 		}
 	}
@@ -224,7 +220,7 @@ int	product_flag;	/* true for product, otherwise sum */
 			return false;
 		}
 	}
-	if (extra_garbage(cp))
+	if (extra_characters(cp))
 		return false;
 	count_down = (end < start);
 	if (fmod(fabs(start - end) / step, 1.0)) {
@@ -571,6 +567,11 @@ output_options()
 	}
 	fprintf(gfp, "autosolve\n");
 
+	if (!autocalc) {
+		fprintf(gfp, "no ");
+	}
+	fprintf(gfp, "autocalc\n");
+
 #if	!SILENT
 	fprintf(gfp, "debug_level = %d\n", debug_level);
 #endif
@@ -614,6 +615,8 @@ output_options()
 		fprintf(gfp, "no ");
 	}
 	fprintf(gfp, "factor_integers\n");
+
+	fprintf(gfp, "special_variable_character = %c\n", special_variable_character);
 }
 
 /*
@@ -680,6 +683,19 @@ next_option:
 		goto next_option;
 	}
 #endif
+	if (strncasecmp(cp, "special", 7) == 0) {
+		cp = skip_param(cp);
+		if (negate) {
+			special_variable_character = '\0';
+		} else {
+			if (*cp == '\0' || cp[1]) {
+				error(_("Please specify a single character."));
+				return false;
+			}
+			special_variable_character = *cp;
+		}
+		return true;
+	}
 	if (strncasecmp(cp, "columns", 7) == 0) {
 		cp = skip_param(cp);
 		if (negate) {
@@ -699,6 +715,11 @@ next_option:
 		autosolve = !negate;
 		goto next_option;
 	}
+	if (strncasecmp(cp, "autocalc", 8) == 0) {
+		cp = skip_param(cp);
+		autocalc = !negate;
+		goto next_option;
+	}
 	if (strncasecmp(cp, "case", 4) == 0) {
 		cp = skip_param(cp);
 		case_sensitive_flag = !negate;
@@ -765,6 +786,9 @@ char	*cp;
 	if (strncasecmp(cp1, "quit", 4) == 0) {
 		return false;
 	}
+	if (strncasecmp(cp1, "exit", 4) == 0) {
+		return false;
+	}
 	return true;
 }
 
@@ -793,10 +817,7 @@ char	*cp;
 	for (i1 = i; i1 <= j; i1++) {
 		if (exists[i1]) {
 			k = next_espace();
-			blt(lhs[k], lhs[i1], n_lhs[i1] * sizeof(token_type));
-			n_lhs[k] = n_lhs[i1];
-			blt(rhs[k], rhs[i1], n_rhs[i1] * sizeof(token_type));
-			n_rhs[k] = n_rhs[i1];
+			copy_espace(i1, k);
 			return_result(k);
 		}
 	}
@@ -841,7 +862,7 @@ int	imag_flag;	/* if true, copy the imaginary part, otherwise copy the real part
 		if (cp == NULL) {
 			return false;
 		}
-		if (extra_garbage(cp))
+		if (extra_characters(cp))
 			return false;
 	}
 	simp_loop(source, nps);
@@ -852,14 +873,10 @@ int	imag_flag;	/* if true, copy the imaginary part, otherwise copy the real part
 	partial_flag = true;
 	n1 = 1;
 	dest[0] = zero_token;
-	has_imag = false;
-	has_real = false;
-	k = 0;
-	for (beg = k; beg < *nps; beg = k, k++) {
+	has_imag = has_real = false;
+	for (beg = k = 0; beg < *nps; beg = k, k++) {
 		found_imag = false;
-		for (;; k++) {
-			if (k >= *nps)
-				break;
+		for (; k < *nps; k++) {
 			if (source[k].level == 1 && source[k].kind == OPERATOR
 			    && (source[k].token.operatr == PLUS || source[k].token.operatr == MINUS)) {
 				break;
@@ -925,7 +942,7 @@ char	*cp;
 	int	i;
 	double	count;
 
-	if (extra_garbage(cp))
+	if (extra_characters(cp))
 		return false;
 	trhs[0] = zero_token;
 	n_trhs = 1;
@@ -994,8 +1011,7 @@ char	*cp;
 {
 	int		i, j, k;
 	long		v, last_v, it_v = 0;
-	long		counter;
-	long		counter_max;
+	long		counter, counter_max;
 	sign_array_type	sa_mark;
 	sign_array_type	sa_value;
 	long		l, iterations = 1;
@@ -1083,7 +1099,7 @@ char	*cp;
 			n_tes = n_trhs;
 		}
 	} else {
-		simp_side(trhs, &n_trhs);
+		calc_simp(trhs, &n_trhs);
 	}
 	for (j = 0; j < ARR_CNT(sa_mark); j++)
 		sa_mark[j] = false;
@@ -1097,9 +1113,8 @@ char	*cp;
 			k++;
 		}
 	}
-	counter_max = (1L << k) - 1;
-	counter = 0;
-	for (; counter <= counter_max; counter++) {
+	counter_max = (1L << k) - 1L;
+	for (counter = 0; counter <= counter_max; counter++) {
 		blt(tlhs, trhs, n_trhs * sizeof(token_type));
 		n_tlhs = n_trhs;
 		for (j = 0, k = 0; j < ARR_CNT(sa_mark); j++) {
@@ -1195,7 +1210,7 @@ int	*diff_signp;
 
 	rv = se_compare(rhs[i], n_rhs[i], rhs[i], n_rhs[i], diff_signp);
 	if (!rv || *diff_signp) {
-		error(_("Error in compare function or too many terms to compare!"));
+		error(_("Error in compare function or too many terms to compare."));
 		return false;
 	}
 	sign_flag = true;
@@ -1213,7 +1228,7 @@ int	i, j;
 
 	rv = se_compare(lhs[i], n_lhs[i], lhs[i], n_lhs[i], &diff_sign);
 	if (!rv || diff_sign) {
-		error(_("Error in compare function or too many terms to compare!"));
+		error(_("Error in compare function or too many terms to compare."));
 		return false;
 	}
 	sign_flag = true;
@@ -1277,7 +1292,7 @@ char	*cp;
 				fprintf(gfp, _("Expressions are identical.\n"));
 				return true;
 			}
-			fprintf(gfp, _("Expressions may differ.\n"));
+			fprintf(gfp, _("Expressions differ.\n"));
 			return false;
 		}
 		error(_("Cannot compare an expression with an equation."));
@@ -1338,7 +1353,7 @@ char	*cp;
 		fprintf(gfp, _("Equations are identical.\n"));
 		return true;
 	}
-	fprintf(gfp, _("Equations may differ.\n"));
+	fprintf(gfp, _("Equations differ.\n"));
 	return false;
 times_neg1:
 	if (!diff_sign && lhs[i][0].token.variable == lhs[j][0].token.variable) {
@@ -1376,7 +1391,7 @@ char	*cp;
 		if (cp == NULL) {
 			return false;
 		}
-		if (extra_garbage(cp))
+		if (extra_characters(cp))
 			return false;
 	}
 	i = next_espace();
@@ -1735,7 +1750,7 @@ char	*cp;
 	i = cur_equation;
 	if (strcmp_tospace(cp, "constants") == 0) {
 		cp = skip_param(cp);
-		if (extra_garbage(cp))
+		if (extra_characters(cp))
 			return false;
 		subst_constants(lhs[i], &n_lhs[i]);
 		subst_constants(rhs[i], &n_rhs[i]);
@@ -1771,14 +1786,14 @@ char	*cp;
 	last_v = 0;
 	for (;;) {
 		v = -1;
-		for (j = 0; j < n_lhs[i]; j++) {
+		for (j = 0; j < n_lhs[i]; j += 2) {
 			if (lhs[i][j].kind == VARIABLE) {
 				if (lhs[i][j].token.variable > last_v
 				    && (v == -1 || lhs[i][j].token.variable < v))
 					v = lhs[i][j].token.variable;
 			}
 		}
-		for (j = 0; j < n_rhs[i]; j++) {
+		for (j = 0; j < n_rhs[i]; j += 2) {
 			if (rhs[i][j].kind == VARIABLE) {
 				if (rhs[i][j].token.variable > last_v
 				    && (v == -1 || rhs[i][j].token.variable < v))
@@ -1809,7 +1824,7 @@ char	*cp;
 				if ((cp1 = parse_section(scratch, &n, cp1)) == NULL || n <= 0) {
 					return false;
 				}
-				if (extra_garbage(cp1))
+				if (extra_characters(cp1))
 					return false;
 				goto do_this;
 			}
@@ -1992,7 +2007,7 @@ int
 quit_cmd(cp)
 char	*cp;
 {
-	if (extra_garbage(cp))
+	if (extra_characters(cp))
 		return false;
 	exit_program(0);
 	return false;
@@ -2009,7 +2024,6 @@ char	*cp;
 {
 	int	rv;
 	FILE	*fp;
-	jmp_buf	save_save;
 	char	buf[MAX_CMD_LEN];
 
 	if (*cp == '\0') {
@@ -2022,31 +2036,50 @@ char	*cp;
 	if (fp == NULL) {
 		fp = fopen(cp, "r");
 		if (fp == NULL) {
-			error(_("Can't open file."));
+			error(_("Can't open input file."));
 			return false;
 		}
 	}
+	rv = read_sub(fp);
+	if (rv) {
+		printf(_("Read operation aborted.\n"));
+	}
+	fclose(fp);
+	return(!rv);
+}
+
+/*
+ * Read and process Mathomatic input from a file pointer.
+ *
+ * Return zero if no error, non-zero if aborted.
+ */
+int
+read_sub(fp)
+FILE	*fp;
+{
+	int	rv;
+	jmp_buf	save_save;
+	char	*cp;
+
 	blt(save_save, jmp_save, sizeof(jmp_save));
 	if ((rv = setjmp(jmp_save)) != 0) {
 		clean_up();
-		if (rv == 14)
+		if (rv == 14) {
 			error(_("Expression too large."));
-		printf(_("Read operation aborted.\n"));
-		goto end_read;
-	}
-	while ((cp = fgets((char *) tlhs, n_tokens * sizeof(token_type), fp)) != NULL) {
-		default_color();
-		input_column = printf("%d%s", cur_equation + 1, html_flag ? HTML_PROMPT : PROMPT);
-		printf("%s", cp);
-		set_error_level(cp);
-		if (!process(cp)) {
-			longjmp(jmp_save, 3);
+		}
+	} else {
+		while ((cp = fgets((char *) tlhs, n_tokens * sizeof(token_type), fp)) != NULL) {
+			default_color();
+			input_column = printf("%d%s", cur_equation + 1, html_flag ? HTML_PROMPT : PROMPT);
+			printf("%s", cp);
+			set_error_level(cp);
+			if (!process(cp)) {
+				longjmp(jmp_save, 3);
+			}
 		}
 	}
-end_read:
 	blt(jmp_save, save_save, sizeof(jmp_save));
-	fclose(fp);
-	return(!rv);
+	return rv;
 }
 #endif
 
@@ -2065,7 +2098,7 @@ char	*cp;
 
 	if (*cp == '\0') {
 #if	CYGWIN
-		my_strlcpy(tmp_file, "mathxxx.tmp", sizeof(tmp_file));
+		my_strlcpy(tmp_file, "math.tmp", sizeof(tmp_file));
 		fp = fopen(tmp_file, "w+");
 		if (fp == NULL) {
 			error(_("Can't create temporary file."));
