@@ -1,7 +1,7 @@
 /*
  * Floating point complex number routines specifically for Mathomatic.
  *
- * Copyright (C) 1987-2006 George Gesslein II.
+ * Copyright (C) 1987-2007 George Gesslein II.
  */
 
 #include "includes.h"
@@ -176,7 +176,7 @@ start_over:
 		goto start_over;
 	}
 	if (modified) {
-		debug_string(0, "Warning: complex number roots approximated, result may be inaccurate or incorrect.");
+		debug_string(0, "Warning: complex number root approximated, result may be inaccurate.");
 	}
 	return modified;
 }
