@@ -1,7 +1,7 @@
 /*
  * Mathomatic differentiation routines and commands.
  *
- * Copyright (C) 1987-2006 George Gesslein II.
+ * Copyright (C) 1987-2007 George Gesslein II.
  */
 
 #include "includes.h"
@@ -402,7 +402,7 @@ char	*cp;
 		if (cp == NULL) {
 			return false;
 		}
-		if (extra_garbage(cp)) {
+		if (extra_characters(cp)) {
 			return false;
 		}
 	}
@@ -532,20 +532,16 @@ char	*cp;
 		if ((cp = parse_section(lhs[our], &our_nlhs, cp)) == NULL || our_nlhs <= 0) {
 			return false;
 		}
-		if (extra_garbage(cp))
+		if (extra_characters(cp))
 			return false;
 	} else {
 #if	!SILENT
 		list_var(v, 0);
-		printf(_("Taylor series expansion about (%s) = point.\n"), var_str);
+		printf(_("Taylor series expansion about %s = point.\n"), var_str);
 #endif
-		for (;;) {
-			my_strlcpy(prompt_str, _("Enter point: "), sizeof(prompt_str));
-			if (!get_expr(lhs[our], &our_nlhs)) {
-				error(_("Enter \"0\" if you don't know."));
-				continue;
-			}
-			break;
+		my_strlcpy(prompt_str, _("Enter point: "), sizeof(prompt_str));
+		if (!get_expr(lhs[our], &our_nlhs)) {
+			return false;
 		}
 	}
 	if (order < 0) {
@@ -652,6 +648,9 @@ loop_again:
 			goto loop_again;
 		}
 	}
+#if	!SILENT
+	printf(_("%ld derivative%s applied.\n"), n, (n == 1) ? "" : "s");
+#endif
 	if (n_rhs[cur_equation]) {
 		n_lhs[i] = n_lhs[cur_equation];
 	}
@@ -679,7 +678,7 @@ char	*cp;
 		return false;
 	}
 	if (n_rhs[cur_equation] == 0) {
-/* make expression into equation: */
+/* make expression into an equation: */
 		blt(rhs[cur_equation], lhs[cur_equation], n_lhs[cur_equation] * sizeof(token_type));
 		n_rhs[cur_equation] = n_lhs[cur_equation];
 		n_lhs[cur_equation] = 1;
@@ -727,29 +726,24 @@ char	*cp;
 			return false;
 		}
 	}
-	if (extra_garbage(cp))
+	if (extra_characters(cp))
 		return false;
 /* copy to a new equation space and work on the copy: */
 	i = next_espace();
-	blt(lhs[i], lhs[cur_equation], n_lhs[cur_equation] * sizeof(token_type));
-	n_lhs[i] = n_lhs[cur_equation];
-	blt(rhs[i], rhs[cur_equation], n_rhs[cur_equation] * sizeof(token_type));
-	n_rhs[i] = n_rhs[cur_equation];
+	copy_espace(cur_equation, i);
 	cur_equation = i;
 /* see if the limit expression contains infinity: */
 	simp_loop(tes, &n_tes);
 	infinity_flag = exp_contains_infinity(tes, n_tes);
-/* fully simplify the current equation and group denominators together: */
+/* first fully simplify the current equation and group denominators together: */
 	simpa_side(rhs[cur_equation], &n_rhs[cur_equation], false);
 	group_proc(rhs[cur_equation], &n_rhs[cur_equation]);
-	found_den = false;
-	complex_den = false;
+	found_den = complex_den = false;
 	for (j = 1; j < n_rhs[cur_equation]; j += 2) {
 		switch (rhs[cur_equation][j].token.operatr) {
 		case DIVIDE:
 		case MODULUS:
-			num = false;
-			den = false;
+			num = den = false;
 			level = rhs[cur_equation][j].level;
 			for (k = j + 2;; k += 2) {
 				if (rhs[cur_equation][k-1].kind == VARIABLE
@@ -782,14 +776,15 @@ char	*cp;
 			break;
 		}
 	}
-	if (!found_den || (infinity_flag && !complex_den)) {
+	if (infinity_flag ? (!complex_den && found_den) : (!found_den)) {
 /* in this case, just substitute the limit variable with the limit expression and simplify: */
 		subst_var_with_exp(rhs[cur_equation], &n_rhs[cur_equation], tes, n_tes, v);
 		simpa_side(rhs[cur_equation], &n_rhs[cur_equation], false);
 		return return_result(cur_equation);
 	}
-	if (infinity_flag) {
-/* To take the limit to infinity, */
+	debug_string(1, "Taking the limit by solving...");
+	if (n_tes == 1 && tes[0].kind == CONSTANT && tes[0].token.constant == HUGE_VAL) {
+/* To take the limit to positive infinity, */
 /* replace infinity with zero and replace the limit variable with its reciprocal: */
 		n_tes = 1;
 		tes[0] = zero_token;
