@@ -1,7 +1,7 @@
 /*
- * Mathomatic global variable definitions.
+ * Mathomatic global variable extern definitions, from file "globals.c".
  *
- * Copyright (C) 1987-2006 George Gesslein II.
+ * Copyright (C) 1987-2007 George Gesslein II.
  */
 
 extern int		n_tokens;
@@ -40,6 +40,9 @@ extern int		screen_columns;
 extern int		screen_rows;
 extern int		finance_option;
 extern int		autosolve;
+extern int		autocalc;
+extern char		special_variable_character;
+extern int		e_i_as_constants;
 extern int		debug_level;
 extern int		domain_check;
 extern int		color_flag;
