@@ -1,7 +1,7 @@
 /*
- * Mathomatic constant factorizing routines.
+ * Mathomatic floating point constant factorizing routines.
  *
- * Copyright (C) 1987-2006 George Gesslein II.
+ * Copyright (C) 1987-2007 George Gesslein II.
  */
 
 #include "includes.h"
@@ -11,7 +11,7 @@ static int	fc_recurse();
 
 /* The following data is used to factor integers: */
 static double nn, vv;
-static double sq[] = {		/* Additive array that skips over multiples of 2, 3, 5, and 7. */
+static double skip_multiples[] = {	/* Additive array that skips over multiples of 2, 3, 5, and 7. */
 	10, 2, 4, 2, 4, 6, 2, 6,
 	 4, 2, 4, 6, 6, 2, 6, 4,
 	 2, 6, 4, 6, 8, 4, 2, 4,
@@ -53,8 +53,8 @@ double	start;
 	try_factor(7.0);
 	d = 1.0;
 	while (d <= vv) {
-		for (i = 0; i < ARR_CNT(sq); i++) {
-			d += sq[i];
+		for (i = 0; i < ARR_CNT(skip_multiples); i++) {
+			d += skip_multiples[i];
 			try_factor(d);
 		}
 	}
@@ -62,7 +62,7 @@ double	start;
 		try_factor(nn);
 	}
 	if (start != multiply_out_unique()) {
-		error("Error factoring integers.");
+		error("Internal error factoring integers.");
 		return false;
 	}
 	return true;
@@ -132,9 +132,9 @@ display_unique()
 }
 
 /*
- * Factor integers in an expression.
+ * Factor integers in an equation side.
  *
- * Return true if expression was modified.
+ * Return true if equation side was modified.
  */
 int
 factor_int(equation, np)
@@ -208,12 +208,15 @@ int		*np;
 factor_int_sub(n)
 int	n;	/* equation space number */
 {
+	if (n_lhs[n] <= 0)
+		return;
 	factor_int(lhs[n], &n_lhs[n]);
 	factor_int(rhs[n], &n_rhs[n]);
 }
 
 /*
- * Factor constants in equation side.
+ * Neatly factor out coefficients in additive expressions in an equation side.
+ * For example: (2*x + 4*y + 6) becomes 2*(x + 2*y + 3).
  *
  * This routine is often necessary because the expression compare (se_compare())
  * does not return a multiplier (except for +/-1.0).
@@ -236,9 +239,9 @@ int	n;	/* equation space number */
  */
 int
 factor_constants(equation, np, level_code)
-token_type	*equation;
-int		*np;
-int		level_code;
+token_type	*equation;	/* pointer to the beginning of equation side */
+int		*np;		/* pointer to length of equation side */
+int		level_code;	/* see above */
 {
 	if (level_code > 2)
 		return false;
