@@ -1,7 +1,7 @@
 /*
  * General floating point GCD routine and associated code for Mathomatic.
  *
- * Copyright (C) 1987-2006 George Gesslein II.
+ * Copyright (C) 1987-2007 George Gesslein II.
  */
 
 #include "includes.h"
@@ -9,6 +9,7 @@
 /*
  * Return the Greatest Common Divisor (GCD) of doubles "d1" and "d2",
  * by using the Euclidean GCD algorithm.
+ *
  * The GCD is defined as the largest positive number that evenly divides both "d1" and "d2".
  * Will work with non-integers, but there may be some floating point error.
  * Always works correctly with integers.
@@ -42,7 +43,7 @@ double	d1, d2;
 	}
 	d = larger * epsilon;
 	if (d >= divisor) {
-		return 0.0;
+		return 0.0;	/* result would be too inaccurate */
 	}
 	for (count = 1; count < 50; count++) {
 		r1 = fmod(larger, divisor);
@@ -58,7 +59,7 @@ double	d1, d2;
 }
 
 /*
- * Round a double to the nearest integer.
+ * Return a floating point double rounded to the nearest integer.
  */
 double
 my_round(d1)
@@ -80,11 +81,10 @@ double	d1;	/* value to round */
  * numerator = d / divisor
  * denominator = 1.0 / divisor
  *
- * Return true with integer numerator and denominator
- * if conversion was successful.
+ * Return true with integer numerator and denominator if conversion was successful.
  * Otherwise return false with numerator = "d" and denominator = "1.0".
  *
- * True return indicates "d" is rational, otherwise "d" is irrational.
+ * True return indicates "d" is rational and finite, otherwise "d" is irrational.
  */
 int
 f_to_fraction(d, numeratorp, denominatorp)
@@ -98,44 +98,49 @@ double	*denominatorp;	/* returned denominator */
 
 	*numeratorp = d;
 	*denominatorp = 1.0;
-/* first, see if it is an integer, or very close to an integer: */
-	if (fmod(d, 1.0) == 0.0)
+	if (!isfinite(d)) {
+		return false;
+	}
+/* see if "d" is an integer, or very close to an integer: */
+	if (fmod(d, 1.0) == 0.0) {
 		return true;
-#if	false	/* set true for more integer oriented math */
+	}
+#if	true	/* set true for more integer oriented math */
 	k3 = fabs(d) * epsilon;
 #else
 	k3 = fabs(d) * small_epsilon;
 #endif
 	k4 = my_round(d);
-	if (k4 != 0.0 && fabs(k4 - d) < k3) {
+	if (k4 != 0.0 && fabs(k4 - d) <= k3) {
 		*numeratorp = k4;
 		return true;
 	}
+/* try to convert non-integer, floating point value in "d" to a fraction: */
 	if ((divisor = gcd(1.0, d)) > epsilon) {
-		numerator = d / divisor;
-		denominator = 1.0 / divisor;
-		numerator = my_round(numerator);
-		denominator = my_round(denominator);
+		numerator = my_round(d / divisor);
+		denominator = my_round(1.0 / divisor);
+/* don't allow more than 11 digits in the numerator or denominator: */
 		if (fabs(numerator) >= 1.0e12)
 			return false;
-		if (denominator >= 1.0e12)
+		if (denominator >= 1.0e12 || denominator < 2.0)
 			return false;
-		if (denominator < 1.5)
-			return false;
-		divisor = gcd(numerator, denominator);	/* make sure the result is a fully reduced fraction */
-		if (fmod(divisor, 1.0) != 0.0) {
-			error("Error in gcd() function!");
-			return false;
-		}
+/* make sure the result is a fully reduced fraction: */
+		divisor = gcd(numerator, denominator);
 		if (divisor > 1.0) {	/* shouldn't happen, but just in case */
-			numerator = numerator / divisor;
-			denominator = denominator / divisor;
+			numerator /= divisor;
+			denominator /= divisor;
 		}
 		k3 = (numerator / denominator);
 		k4 = d;
 		if (fabs(k3 - k4) > (small_epsilon * fabs(k3))) {
 			return false;	/* result is too inaccurate */
 		}
+		if (fmod(numerator, 1.0)) {
+			error("Internal error: Numerator not integral.");
+		}
+		if (fmod(denominator, 1.0)) {
+			error("Internal error: Denominator not integral.");
+		}
 		*numeratorp = numerator;
 		*denominatorp = denominator;
 		return true;
