@@ -1,9 +1,9 @@
 /*
- * Mathomatic global variables.
+ * Mathomatic global variables and arrays.
  *
- * Copyright (C) 1987-2006 George Gesslein II.
+ * Copyright (C) 1987-2007 George Gesslein II.
  *
- * All global variables for Mathomatic are defined here and duplicated in "externs.h".
+ * Most global variables for Mathomatic are defined here and duplicated in "externs.h".
  */
 
 #include "includes.h"
@@ -32,28 +32,31 @@ token_type	zero_token;			/* the constant 0.0 as a token */
 token_type	one_token;			/* the constant 1.0 as a token */
 
 /* set options */
-int		precision = 14;			/* the display precision (number of digits) */
-int		case_sensitive_flag = true;	/* "set case_sensitive" flag */
-int		factor_int_flag;		/* factor integers when displaying expressions */
-int		display2d = true;		/* "set display2d" flag */
-int		preserve_roots = true;		/* set option to preserve 2^.5, etc. */
-int		true_modulus;			/* true for mathematically correct modulus */
-int		screen_columns = STANDARD_SCREEN_COLUMNS;	/* screen width */
-int		screen_rows = STANDARD_SCREEN_ROWS;		/* screen height */
-int		finance_option;			/* for displaying dollars and cents */
-int		autosolve = true;		/* Allows solving by just typing the variable name at the prompt */
+int		precision = 14;				/* the display precision for doubles (number of digits) */
+int		case_sensitive_flag = true;		/* "set case_sensitive" flag */
+int		factor_int_flag;			/* factor integers when displaying expressions */
+int		display2d = true;			/* "set display2d" flag */
+int		preserve_roots = true;			/* set option to preserve roots like (2^.5) */
+int		true_modulus;				/* true for mathematically correct modulus */
+int		screen_columns = STANDARD_SCREEN_COLUMNS;	/* screen width of the terminal; 0 = infinite */
+int		screen_rows = STANDARD_SCREEN_ROWS;		/* screen height of the terminal; 0 = infinite */
+int		finance_option;				/* for displaying dollars and cents */
+int		autosolve = true;			/* Allows solving by typing the variable name at the prompt */
+int		autocalc = true;			/* Allows automatically calculating a numerical expression */
+char		special_variable_character = '\\';	/* user defined special character for variable names */
+int		e_i_as_constants = true;		/* set true to allow "e" and "i" as special constants */
 #if	!SILENT
-int		debug_level;			/* current debug level */
+int		debug_level;				/* current debug level */
 #endif
 
 /* variables having to do with color mode */
 int		color_flag = true;		/* true for color mode */
 int		bold_colors;			/* true for bold colors */
-int		cur_color = -1;			/* current color */
+int		cur_color = -1;			/* current color on the terminal */
 
 /* epsilon constants */
 double		small_epsilon	= 0.000000000000005;	/* for small round-off errors */
-double		epsilon		= 0.00000000000005;	/* for larger accumulated round-off errors */
+double		epsilon		= 0.00000000000005;	/* for larger, accumulated round-off errors */
 
 /* string variables */
 char		*var_names[MAX_VAR_NAMES];	/* storage for long variable names */
@@ -83,7 +86,7 @@ int		high_prec;		/* flag to output constants in higher precision (used when savi
 int		input_column;		/* current column number on the screen at the beginning of a parse */
 int		sign_flag;		/* true when all "sign" variables are to compare equal */
 int		domain_check;		/* flag to track domain errors in the pow() function */
-int		approximate_roots;	/* true if in calculate command (force approximations) */
+int		approximate_roots;	/* true if in calculate command (force approximations of roots like (2^.5)) */
 
 /* library variables go here */
 #if	SILENT
