@@ -3,7 +3,7 @@
  *
  * Everything that depends on the command table goes here.
  *
- * Copyright (C) 1987-2006 George Gesslein II.
+ * Copyright (C) 1987-2007 George Gesslein II.
  */
 
 #include "includes.h"
@@ -164,26 +164,30 @@ char	*cp;
 				goto return_ok;
 			}
 #if	!LIBRARY
-			/* the numerical input calculation */
-			for (i = 0; i < n_lhs[n]; i += 2) {
-				if (lhs[n][i].kind == VARIABLE && (lhs[n][i].token.variable & VAR_MASK) > SIGN) {
-					goto return_ok;		/* not numerical (contains a variable) */
+			if (autocalc) {
+				/* the numerical input calculation */
+				for (i = 0; i < n_lhs[n]; i += 2) {
+					if (lhs[n][i].kind == VARIABLE && (lhs[n][i].token.variable & VAR_MASK) > SIGN) {
+						goto return_ok;		/* not numerical (contains a variable) */
+					}
 				}
+				/* make the expression an equation */
+				blt(rhs[n], lhs[n], n_lhs[n] * sizeof(token_type));
+				n_rhs[n] = n_lhs[n];
+				lhs[n][0].level = 1;
+				lhs[n][0].kind = VARIABLE;
+				parse_var(&lhs[n][0].token.variable, "answer");
+				n_lhs[n] = 1;
+				i = cur_equation;
+				/* make the current equation and run the calculate command on it */
+				cur_equation = n;
+				calculate_cmd("");
+				cur_equation = i;
+				/* delete from memory */
+				n_lhs[n] = 0;
+				n_rhs[n] = 0;
+				return true;
 			}
-			/* make the expression an equation */
-			blt(rhs[n], lhs[n], n_lhs[n] * sizeof(token_type));
-			n_rhs[n] = n_lhs[n];
-			lhs[n][0].level = 1;
-			lhs[n][0].kind = VARIABLE;
-			parse_var(&lhs[n][0].token.variable, "answer");
-			n_lhs[n] = 1;
-			i = cur_equation;
-			cur_equation = n;
-			calculate_cmd("");
-			cur_equation = i;
-			n_lhs[n] = 0;
-			n_rhs[n] = 0;
-			return true;
 #endif
 		}
 return_ok:
@@ -431,8 +435,16 @@ char	*cp;
 		if (flag)
 			return true;
 		if (strncasecmp(cp, "usage", cp1 - cp) == 0) {
-			for (i = 0; i < ARR_CNT(com_list); i++) {
+			P("Command Usage Syntax");
+			P("--------------------");
+			for (i = 0, row = 3; i < ARR_CNT(com_list); i++) {
 				fprintf(gfp, "%s %s\n", com_list[i].name, com_list[i].usage);
+				row++;
+				if (gfp == stdout && screen_rows && row >= (screen_rows - 1)) {
+					row = 1;
+					if (!pause_cmd(""))
+						return false;
+				}
 			}
 			return true;
 		}
@@ -473,9 +485,9 @@ char	*cp;
 			P("Variables consist of any combination of letters, digits, and underscores (_).");
 			P("Constants always start with a digit (0..9) or a period.");
 			P("Predefined constants and variables follow:\n");
-			P("    e# - the universal constant e (2.7182818284...).");
-			P("    pi - the universal constant pi (3.1415926535...).");
-			P("    i# - imaginary number (square root of -1).");
+			P("    e or e# - the universal constant e (2.7182818284...).");
+			P("    pi or pi# - the universal constant pi (3.1415926535...).");
+			P("    i or i# - imaginary number (square root of -1).");
 			P("    sign, sign1, sign2, sign3, ... - may only be +1 or -1.");
 			P("    integer - may be any integer.");
 			P("    inf - infinity constant.\n");
@@ -498,7 +510,7 @@ char	*cp;
 	}
 	/* default help text: */
 	P("This help command is provided as a quick reference.");
-	P("Type \"help all\" for a summary of the all commands.");
+	P("Type \"help all\" for a summary and syntax of the all commands.");
 	P("Type \"help usage\" for syntax of all commands.");
 	P("Type \"help expressions\" for help with entering expressions.");
 	P("Type \"help geometry\" for some commonly used geometric formulas.");
@@ -511,7 +523,7 @@ char	*cp;
 		for (; j > 0; j--)
 			fprintf(gfp, " ");
 	}
-	P("\n\nTo select an equation in memory, just enter the equation number.");
-	P("To solve the current equation, type the variable name at the prompt.");
+	P("\n\nTo select an equation in memory, type the equation number at the main prompt.");
+	P("To solve the current equation, type the variable name at the main prompt.");
 	return true;
 }
