@@ -2,22 +2,32 @@
  * Mathomatic expression and equation display routines.
  * Color mode routines, too.
  *
- * Copyright (C) 1987-2006 George Gesslein II.
+ * Copyright (C) 1987-2007 George Gesslein II.
  */
 
 #include "includes.h"
 
+#define	EQUATE_STRING	" = "	/* string displayed between the LHS and RHS of equations */
+
+#define	APPEND(str)	{ if (string) { strcpy(&string[len], str); } len += strlen(str); }
+
 static int	flist();
 static int	flist_recurse();
 
 /* ANSI color code array */
 static int	carray[] = {
-	32, 33, 31, 35, 34, 36, 37
+	32,	/* green (default color) */
+	33,	/* yellow */
+	31,	/* red */
+	35,	/* magenta */
+	34,	/* blue */
+	36,	/* cyan */
+	37	/* white */
 };
 
 /* HTML color array */
 static char	*html_carray[] = {
-	"#00FF00",
+	"#00FF00",	/* bright green (default color) */
 	"#FFFF00",
 	"#FF9000",
 	"#FF0000",
@@ -26,6 +36,10 @@ static char	*html_carray[] = {
 	"#0000FF"
 };
 
+/* global variables for flist() */
+int	cur_line;	/* current line */
+int	cur_pos;	/* current position in the current line on the screen */
+
 /*
  * Turn color off if color mode is on.
  */
@@ -97,7 +111,7 @@ int	export_flag;	/* true for exportable format (readable by other math programs)
 	}
 	list_proc(lhs[n], n_lhs[n], export_flag);
 	if (n_rhs[n]) {
-		fprintf(gfp, " = ");
+		fprintf(gfp, EQUATE_STRING);
 		list_proc(rhs[n], n_rhs[n], export_flag);
 	}
 	if (export_flag == 1) {
@@ -154,7 +168,7 @@ int		n2;
 		printf(_("level %d: "), level);
 		list_proc(p1, n1, false);
 		if (p2 && n2 > 0) {
-			printf(" = ");
+			printf(EQUATE_STRING);
 			list_proc(p2, n2, false);
 		}
 		printf("\n");
@@ -236,6 +250,9 @@ int	lang_code;	/* language code */
 		case -1:
 			cp = "%pi";
 			break;
+		case 0:
+			cp = "pi#";
+			break;
 		case 1:
 			cp = "M_PI";
 			break;
@@ -270,13 +287,6 @@ int	lang_code;	/* language code */
 		snprintf(buf, sizeof(buf), "%d", j - 1);
 		strcat(var_str, buf);
 	}
-	if (lang_code > 0) {
-		for (cp = var_str; *cp; cp++) {
-			if (*cp == '\\') {
-				*cp = '_';
-			}
-		}
-	}
 	return(strlen(var_str));
 }
 
@@ -390,7 +400,7 @@ int	export_flag;	/* flag for exportable format (usually false) */
 
 	len = list_string_sub(lhs[n], n_lhs[n], NULL, export_flag);
 	if (n_rhs[n]) {
-		len += 3;
+		len += strlen(EQUATE_STRING);
 		len += list_string_sub(rhs[n], n_rhs[n], NULL, export_flag);
 	}
 	len += 2;
@@ -401,7 +411,7 @@ int	export_flag;	/* flag for exportable format (usually false) */
 	}
 	list_string_sub(lhs[n], n_lhs[n], cp, export_flag);
 	if (n_rhs[n]) {
-		strcat(cp, " = ");
+		strcat(cp, EQUATE_STRING);
 		list_string_sub(rhs[n], n_rhs[n], &cp[strlen(cp)], export_flag);
 	}
 	if (export_flag == 1) {
@@ -450,8 +460,6 @@ int		n;		/* length of expression */
 char		*string;	/* buffer or NULL pointer */
 int		export_flag;
 {
-#define	APPEND(str)	{ if (string) { strcpy(&string[len], str); } len += strlen(str); }
-
 	int	i, j, k, i1;
 	int	min1;
 	int	cur_level;
@@ -575,7 +583,7 @@ int	int_flag;	/* integer arithmetic flag */
 
 /*
  * Output C, Java, or Python code for an expression.
- * Expression may be modified.
+ * Expression might be modified by this function.
  *
  * Return length of output (number of characters).
  */
@@ -603,8 +611,7 @@ int		int_flag;	/* integer arithmetic flag, should work with any language */
 		for (i1 = 1; i1 <= k; i1++) {
 			if (j > 0) {
 				cur_level--;
-				fprintf(gfp, ")");
-				len++;
+				len += fprintf(gfp, ")");
 			} else {
 				cur_level++;
 				for (i2 = i + 1; i2 < *np && equation[i2].level >= cur_level; i2 += 2) {
@@ -636,8 +643,7 @@ int		int_flag;	/* integer arithmetic flag, should work with any language */
 						break;
 					}
 				}
-				fprintf(gfp, "(");
-				len++;
+				len += fprintf(gfp, "(");
 			}
 		}
 		switch (equation[i].kind) {
@@ -706,11 +712,8 @@ int		int_flag;	/* integer arithmetic flag, should work with any language */
 	return len;
 }
 
-int	cur_line;	/* current line */
-int	cur_pos;	/* current position in the current line on the screen */
-
 /*
- * Display an expression or equation in multi-line fraction format.
+ * Display an equation space in multi-line fraction format.
  * Use color if available.
  *
  * Return true if successful.
@@ -719,7 +722,6 @@ int
 flist_sub(n)
 int	n;	/* equation space number */
 {
-	int	i;
 	int	sind;
 	char	buf[30];
 	int	len, len2, len3;
@@ -736,7 +738,7 @@ int	n;	/* equation space number */
 	sind = n_rhs[n];
 	len += flist(lhs[n], n_lhs[n], false, 0, &max_line, &min_line);
 	if (n_rhs[n]) {
-		len += 3;
+		len += strlen(EQUATE_STRING);
 make_smaller:
 		len2 = flist(rhs[n], sind, false, 0, &high, &low);
 		if (screen_columns && gfp == stdout && (len + len2) >= screen_columns && sind > 0) {
@@ -765,27 +767,25 @@ make_smaller:
 		return list1_sub(n, false);
 	}
 	fprintf(gfp, "\n");
-	for (i = max_line; i >= min_line; i--) {
-		cur_line = i;
-		cur_pos = 0;
-		if (i == 0) {
+	for (cur_line = max_line; cur_line >= min_line; cur_line--) {
+		pos = cur_pos = 0;
+		if (cur_line == 0) {
 			cur_pos += fprintf(gfp, "%s", buf);
 		}
-		pos = strlen(buf);
+		pos += strlen(buf);
 		pos += flist(lhs[n], n_lhs[n], true, pos, &high, &low);
 		if (n_rhs[n]) {
-			if (i == 0) {
-				cur_pos += fprintf(gfp, "%s", " = ");
+			if (cur_line == 0) {
+				cur_pos += fprintf(gfp, "%s", EQUATE_STRING);
 			}
-			pos += 3;
-			flist(rhs[n], sind, true, pos, &high, &low);
+			pos += strlen(EQUATE_STRING);
+			pos += flist(rhs[n], sind, true, pos, &high, &low);
 		}
 		fprintf(gfp, "\n");
 	}
 	if (sind < n_rhs[n]) {
 		fprintf(gfp, "\n");
-		for (i = max2_line; i >= min2_line; i--) {
-			cur_line = i;
+		for (cur_line = max2_line; cur_line >= min2_line; cur_line--) {
 			cur_pos = 0;
 			flist(&rhs[n][sind], n_rhs[n] - sind, true, 0, &high, &low);
 			fprintf(gfp, "\n");
