@@ -2,7 +2,7 @@
  * This file contains main() and the startup code for Mathomatic.
  * Leave this file out when compiling for the symbolic math library.
  *
- * Copyright (C) 1987-2006 George Gesslein II.
+ * Copyright (C) 1987-2007 George Gesslein II.
  *
  * Output to stderr is only done in this file.  The rest of the code
  * should not output to stderr.
@@ -28,6 +28,9 @@
  * Any number of TIMES and DIVIDE operators may be on the same
  * level, because they are simple multiplicative class operators.
  * The same for PLUS and MINUS, because they are additive class operators.
+ *
+ * See the file "am.h" to start understanding the Mathomatic code and
+ * to adjust memory usage.
  */
 
 #if	!LIBRARY
@@ -38,26 +41,6 @@
 static void usage();
 static int set_signals();
 
-#if	CYGWIN
-/*
- * dirname(3) function for Windows.
- */
-char	*
-dirname1(cp)
-char	*cp;
-{
-	int	i;
-
-	i = strlen(cp);
-	while (i >= 0 && cp[i] != '\\' && cp[i] != '/')
-		i--;
-	if (i < 0)
-		return(".");
-	cp[i] = '\0';
-	return(cp);
-}
-#endif
-
 int
 main(argc, argv)
 int	argc;
@@ -70,14 +53,13 @@ char	**argv;
 	char		*cp;
 	double		numerator, denominator;
 	double		multiplier;
-	struct winsize	ws;
 	int		coption = false;
 
 #if	UNIX
-	prog_name = strdup(basename(argv[0]));	/* set prog_name to this executable's filename */
+	prog_name = strdup(basename(argv[0]));		/* set prog_name to this executable's filename */
 #endif
 #if	CYGWIN
-	dir_path = strdup(dirname1(argv[0]));	/* set dir_path to this executable's directory */
+	dir_path = strdup(dirname_win(argv[0]));	/* set dir_path to this executable's directory */
 #endif
 	/* initialize the global variables */
 	init_gvars();
@@ -87,7 +69,7 @@ char	**argv;
 	while ((i = getopt(argc, argv, "qrtchuvm:")) >= 0) {
 		switch (i) {
 		case 'c':
-			coption = true;
+			coption++;
 			break;
 		case 'h':
 			readline_enabled = false;
@@ -96,7 +78,7 @@ char	**argv;
 		case 'm':
 			multiplier = strtod(optarg, &cp) * DEFAULT_N_TOKENS;
 			if (*cp || multiplier < 100 || multiplier >= (INT_MAX / 3)) {
-				fprintf(stderr, _("Invalid memory size multiplier specified.\n"));
+				fprintf(stderr, _("%s: Invalid memory size multiplier specified.\n"), prog_name);
 				exit(1);
 			}
 			n_tokens = (int) multiplier;
@@ -125,17 +107,7 @@ char	**argv;
 		screen_columns = 0;
 		screen_rows = 0;
 	} else {
-		/* get the screen width and height */
-		ws.ws_col = 0;
-		ws.ws_row = 0;
-		if (ioctl(1, TIOCGWINSZ, &ws) >= 0) {
-			if (ws.ws_col) {
-				screen_columns = ws.ws_col;
-			}
-			if (ws.ws_row) {
-				screen_rows = ws.ws_row;
-			}
-		}
+		get_screen_size();
 	}
 	if (!init_mem()) {
 		fprintf(stderr, _("%s: Not enough memory.\n"), prog_name);
@@ -149,12 +121,12 @@ char	**argv;
 		printf(_("Secure "));
 #endif
 		printf(_("Mathomatic version %s (www.mathomatic.org)\n"), VERSION);
-		printf(_("Copyright (C) 1987-2006 George Gesslein II.\n"));
-		printf(_("%d equation spaces available, %ldKB per equation space.\n\n"),
+		printf(_("Copyright (C) 1987-2007 George Gesslein II.\n"));
+		printf(_("%d equation spaces available, %ldKB per equation space.\n"),
 		    N_EQUATIONS, (long) n_tokens * sizeof(token_type) * 2L / 1000L);
 	}
 #if	!SECURE
-	/* read the options initialization file */
+	/* read the user options initialization file */
 	if (!test_mode && !load_rc()) {
 		fprintf(stderr, _("Error loading set options from \"%s\".\n"), RC_FILE);
 	}
@@ -162,9 +134,12 @@ char	**argv;
 	if (test_mode) {
 		color_flag = false;
 	}
-	if (coption) {
+	if (coption & 1) {
 		color_flag = !color_flag;
 	}
+	if (!test_mode && !quiet_mode && color_flag) {
+		printf(_("%s color mode enabled.\n"), html_flag ? "HTML" : "ANSI");
+	}
 	if ((i = setjmp(jmp_save)) != 0) {
 		/* for error handling */
 		clean_up();
@@ -186,7 +161,7 @@ char	**argv;
 			fprintf(stderr, _("Cannot convert floating point values to fractions.\n"));
 		}
 #if	!SECURE
-		/* read in files on the command line */
+		/* read in files on the command line, exit if error */
 		for (i = optind; i < argc; i++) {
 			if (!read_cmd(argv[i])) {
 				exit_program(1);
@@ -249,6 +224,26 @@ set_signals()
 	return rv;
 }
 
+#if	CYGWIN
+/*
+ * dirname(3) function for Windows.
+ */
+char	*
+dirname_win(cp)
+char	*cp;
+{
+	int	i;
+
+	i = strlen(cp);
+	while (i >= 0 && cp[i] != '\\' && cp[i] != '/')
+		i--;
+	if (i < 0)
+		return(".");
+	cp[i] = '\0';
+	return(cp);
+}
+#endif
+
 #if	!SECURE
 /*
  * Load set options from "~/.mathomaticrc".
@@ -302,13 +297,14 @@ int	sig;
 }
 
 /*
- * Control-C handler.
- * Abruptly quits this program.
+ * Control-C (interrupt) handler.
+ * Abruptly quit this program.
  */
 void
 inthandler(sig)
 int	sig;
 {
+	printf(_("\nControl-C pressed, quitting...\n"));
 	exit_program(1);
 }
 
@@ -320,11 +316,30 @@ void
 alarmhandler(sig)
 int	sig;
 {
-	printf(_("\ntimeout\n"));
+	printf(_("\nTimeout, quitting...\n"));
 	exit_program(1);
 }
 #endif
 
+/*
+ * Get the screen (window) width and height.
+ */
+get_screen_size()
+{
+	struct winsize	ws;
+
+	ws.ws_col = 0;
+	ws.ws_row = 0;
+	if (ioctl(1, TIOCGWINSZ, &ws) >= 0) {
+		if (ws.ws_col > 0 && screen_columns) {
+			screen_columns = ws.ws_col;
+		}
+		if (ws.ws_row > 0 && screen_rows) {
+			screen_rows = ws.ws_row;
+		}
+	}
+}
+
 /*
  * Window resize handler.
  */
@@ -332,18 +347,7 @@ void
 resizehandler(sig)
 int	sig;
 {
-	struct winsize	ws;
-
-	ws.ws_col = 0;
-	ws.ws_row = 0;
-	if (ioctl(1, TIOCGWINSZ, &ws) >= 0) {
-		if (ws.ws_col && screen_columns) {
-			screen_columns = ws.ws_col;
-		}
-		if (ws.ws_row && screen_rows) {
-			screen_rows = ws.ws_row;
-		}
-	}
+	get_screen_size();
 }
 
 /*
@@ -351,7 +355,7 @@ int	sig;
  */
 void
 exit_program(exit_value)
-int	exit_value;
+int	exit_value;	/* zero if OK, non-zero indicates error return */
 {
 	reset_attr();
 	printf("\n");
