@@ -3,22 +3,10 @@
  * algorithm that doesn't use much memory by using a windowing sieve buffer.
  * Works with up to 19 digit numbers.
  *
- * Copyright (C) 2006 George Gesslein II.
- *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
- *
+ * Copyright (C) 2007 George Gesslein II.
+ */
+
+/*
  * Usage: matho-primes [start [stop]] ["pal" [base]]
  *
  * If "pal" is specified, display only palindromic primes of base "base",
@@ -56,7 +44,7 @@
 
 #define	MAX_K_INTEGER	1.0e19L		/* 19 digits for long doubles */
 
-/* memory usage in bytes; can be set to any size; the larger, the faster */
+/* Maximum memory usage in bytes; can be set to any size; the larger, the faster: */
 #define BUFFER_SIZE	2000000
 #if	BUFFER_SIZE >= (INT_MAX / 2) || BUFFER_SIZE <= 0
 #error	"BUFFER_SIZE too large."
@@ -71,7 +59,7 @@ int	get_long_double_int(char *cp, long double *dp);
 long double strtold(const char *nptr, char **endptr);
 
 long double start_value, number, end_value;
-long double sq[] = {		/* Additive array that skips over multiples of 2, 3, 5, and 7. */
+long double skip_multiples[] = {	/* Additive array that skips over multiples of 2, 3, 5, and 7. */
 	10, 2, 4, 2, 4, 6, 2, 6,
 	 4, 2, 4, 6, 6, 2, 6, 4,
 	 2, 6, 4, 6, 8, 4, 2, 4,
@@ -267,8 +255,8 @@ generate_primes()
 		elim_factor(7.0L);
 		ii = 1.0;
 		for (;;) {
-			for (j = 0; j < ARR_CNT(sq); j++) {
-				ii += sq[j];
+			for (j = 0; j < ARR_CNT(skip_multiples); j++) {
+				ii += skip_multiples[j];
 				elim_factor(ii);
 			}
 			if (ii > vv)
