@@ -1,12 +1,13 @@
 /*
- * Mathomatic simplify routines.
+ * Mathomatic simplifying routines.
  *
- * Copyright (C) 1987-2006 George Gesslein II.
+ * Copyright (C) 1987-2007 George Gesslein II.
  */
 
 #include "includes.h"
 
-#define	MAX_COMPARE_TERMS	400	/* Max no. of terms on same level that can be compared, increases stack usage. */
+#define	MAX_COMPARE_TERMS	500	/* Max no. of terms on same level that can be compared, increases stack usage. */
+					/* MAX_COMPARE_TERMS should be approximately (sqrt(DEFAULT_N_TOKENS) * 3) */
 
 static int	org_recurse();
 static int	const_recurse();
@@ -20,8 +21,12 @@ static int	order_recurse();
 simp_sub(n)
 int	n;	/* equation space number to simplify */
 {
+	if (n_lhs[n] <= 0)
+		return;
 	simp_loop(lhs[n], &n_lhs[n]);
-	simp_loop(rhs[n], &n_rhs[n]);
+	if (n_rhs[n]) {
+		simp_loop(rhs[n], &n_rhs[n]);
+	}
 }
 
 /*
@@ -85,6 +90,7 @@ int		*np;
 	approximate_roots = true;
 	subst_constants(equation, np);
 	simp_side(equation, np);
+	factorv(equation, np, IMAGINARY);
 	ufactor(equation, np);
 	factorv(equation, np, IMAGINARY);
 	ufactor(equation, np);
@@ -218,7 +224,7 @@ do_quick:
 	}
 	partial_flag = false;
 	simpb_side(equation, np, true, 1);
-	debug_string(1, "Simplified result before applying polynomial operations:");
+	debug_string(1, "Simplify result before applying polynomial operations:");
 	side_debug(1, equation, *np);
 	for (flag = false;;) {
 		/* divide top and bottom of fractions by any polynomial GCD found */
@@ -258,10 +264,12 @@ do_quick:
 	}
 	poly_factor(equation, np);
 	simpb_side(equation, np, true, 2);
+	debug_string(1, "Result of complete simplify:");
+	side_debug(1, equation, *np);
 }
 
 /*
- * Compare function for qsort(3).
+ * Compare function for qsort(3) within simpb_side().
  */
 static int
 vcmp(p1, p2)
@@ -288,14 +296,13 @@ sort_type	*p1, *p2;
 /*
  * Beauty simplifier.
  * Neat simplify and factor routine.
- * Factors variables in order.
- * "sign" variables first, then by frequency.
+ * Factors variables in order: "sign" variables first, then by frequency.
  */
 simpb_side(equation, np, power_flag, fc_level)
 token_type	*equation;	/* pointer to the beginning of equation side */
 int		*np;		/* pointer to length of equation side */
-int		power_flag;	/* factor powers flag */
-int		fc_level;	/* factor constants code */
+int		power_flag;	/* factor_power() flag */
+int		fc_level;	/* factor constants code, passed to factor_constants() */
 {
 	int		i;
 	int		vc, cnt;
@@ -374,8 +381,8 @@ token_type	*equation;	/* pointer to the beginning of equation side to simplify *
 int		*np;		/* pointer to length of equation side */
 long		v;		/* variable to factor, 0L or MATCH_ANY to factor all variables */
 double		d;		/* factor expressions raised to the power of this if v */
-int		power_flag;	/* factor power flag */
-int		times_flag;	/* factor times flag */
+int		power_flag;	/* factor_power() flag */
+int		times_flag;	/* factor_times() flag */
 int		fc_level;	/* factor constants code, passed to factor_constants() */
 {
 	do {
@@ -410,8 +417,6 @@ int		*np;		/* pointer to length of equation side */
 	int	i;
 	int	rv = false;
 
-	if (*np <= 1)	/* no need to simplify a constant or a variable */
-		return rv;
 	do {
 		do {
 			do {
@@ -463,7 +468,8 @@ int		*np;
 }
 
 /*
- * Combine all like denominators.  Don't factor times.
+ * Combine all like denominators containing variable "v".
+ * Don't call factor_times().
  */
 simp2_divide(equation, np, v, fc_level)
 token_type	*equation;
@@ -491,15 +497,15 @@ int		fc_level;
  * This is the inner-most loop in Mathomatic, make it fast.
  */
 organize(equation, np)
-token_type	*equation;
-int		*np;
+token_type	*equation;	/* equation side pointer */
+int		*np;		/* pointer to length of equation side */
 {
 	if (*np <= 0 || (*np & 1) != 1) {
-		error(_("Organize called with bad expression size!"));
+		error("Internal error: Organize called with bad expression size.");
 		longjmp(jmp_save, 13);
 	}
 	if (*np > n_tokens) {
-		error(_("Array overflow detected!  Expression larger than \"n_tokens\"."));
+		error("Internal error: Expression array overflow detected.");
 		longjmp(jmp_save, 13);
 	}
 	org_recurse(equation, np, 0, 1, NULL);
@@ -538,12 +544,14 @@ int		level, invert;
 }
 
 /*
- * Recursively organize an expression at "level" of parentheses.
+ * Recurse through every sub-expression in "equation", starting at "loc",
+ * moving up levels to "level" of parentheses.
  */
 static int
 org_recurse(equation, np, loc, level, elocp)
-token_type	*equation;
-int		*np, loc, level, *elocp;
+token_type	*equation;	/* equation side pointer */
+int		*np;		/* pointer to length of equation side */
+int		loc, level, *elocp;
 {
 	token_type	*p1, *bp, *ep;
 	int		op, sub_op;
