@@ -1,7 +1,7 @@
 /*
  * Group and combine denominators.
  *
- * Copyright (C) 1987-2006 George Gesslein II.
+ * Copyright (C) 1987-2007 George Gesslein II.
  */
 
 #include "includes.h"
@@ -27,8 +27,8 @@ static void	group_recurse();
  */
 int
 super_factor(equation, np, start_flag)
-token_type	*equation;
-int		*np;
+token_type	*equation;	/* pointer to the beginning of the equation side */
+int		*np;		/* pointer to the length of the equation side */
 int		start_flag;
 {
 	int	rv;
